import json 

from msfabricpysdkcore.fabric_azure_client import FabricAzureClient

class FabricAzureCapacity:
    """Class to represent a item in Microsoft Fabric"""

    def __init__(self, id, name, subscription_id, resource_group_name, type, location, properties, sku, azure_client: FabricAzureClient, tags=None) -> None:
        
        self.id = id
        self.name = name
        self.subscription_id = subscription_id
        self.resource_group_name = resource_group_name
        self.type = type
        self.location = location
        self.properties = properties
        self.sku = sku
        self.tags = tags

        self.azure_client = azure_client

    def __str__(self) -> str:
        """Return a string representation of the fabric azure capacity object"""
        dict_ = {
            'id': self.id,
            'name': self.name,
            'subscription_id': self.subscription_id,
            'resource_group_name': self.resource_group_name,
            'type': self.type,
            'location': self.location,
            'properties': self.properties,
            'sku': self.sku,
            'tags': self.tags
        }
        return json.dumps(dict_, indent=2)
    
    def __repr__(self) -> str:
        return self.__str__()
    
    def from_dict(dict, azure_client):
        """Create FabricAzureCapacity object from dictionary"""
            
        return FabricAzureCapacity(id=dict['id'], name=dict['name'], subscription_id=dict['subscription_id'],
                                   resource_group_name=dict['resource_group_name'],
                                   type=dict['type'],
                                   location=dict['location'],
                                   properties=dict['properties'], sku=dict['sku'],
                                   tags=dict.get('tags', None), azure_client=azure_client)
    
    # Delete

    def delete(self):
        """Delete the capacity"""

        return self.azure_client.delete_capacity(self.subscription_id, self.resource_group_name, self.name)
    
    # Resume

    def resume(self):
        """Resume the capacity"""

        return self.azure_client.resume_capacity(self.subscription_id, self.resource_group_name, self.name)
    
    # Suspend

    def suspend(self):
        """Suspend the capacity"""

        return self.azure_client.suspend_capacity(self.subscription_id, self.resource_group_name, self.name)
    
    # Update

    def update(self, properties_administration=None, sku=None, tags=None):
        """Update the capacity"""

        return self.azure_client.update_capacity(self.subscription_id, self.resource_group_name, self.name, properties_administration=properties_administration, sku=sku, tags=tags)




import json 

class Capacity:
    """Class to represent a capacity in Microsoft Fabric"""


    def __init__(self, id, display_name, sku, region, state):
        """Constructor for the Capacity class
        
        Args:
            id (str): The ID of the capacity
            display_name (str): The display name of the capacity
            sku (str): The SKU of the capacity
            region (str): The region of the capacity
            state (str): The state of the capacity
        
        Returns:
            Capacity: The Capacity object created
        """
        self.id = id
        self.display_name = display_name
        self.sku = sku
        self.region = region
        self.state = state

    def __str__(self):
        """Method to return a string representation of the Capacity object
        
        Returns:
            str: The string representation of the Capacity object
        """
        dic = {
            'id': self.id,
            'display_name': self.display_name,
            'sku': self.sku,
            'region': self.region,
            'state': self.state
        }
        return json.dumps(dic, indent=2)

    def __repr__(self) -> str:
        return self.__str__()

    def from_dict(dic):
        """Method to create a Capacity object from a dictionary
        
        Args:
            dic (dict): The dictionary containing the capacity information
        Returns:
            Capacity: The Capacity object created from the dictionary
        
        """
        if "display_name" not in dic:
            dic["display_name"] = dic["displayName"]
        return Capacity(dic['id'], dic['display_name'], dic['sku'], dic['region'], dic['state'])
        


import logging
from time import sleep, time
from msfabricpysdkcore.coreapi import FabricClientCore
from msfabricpysdkcore.util import logger


class LongRunningOperation:
    """Class to represent a workspace in Microsoft Fabric"""

    _logger: logging.Logger

    def __init__(self, operation_id, core_client: FabricClientCore) -> None:
        """Initialize the LongRunningOperation object"""

        self._logger = logger.getChild(__name__)
        self.operation_id = operation_id
        self.core_client = core_client

        self.state = self.get_operation_state()["status"]

    def get_operation_results(self):
        return self.core_client.get_operation_results(operation_id=self.operation_id)
    
    def get_operation_state(self):
        return self.core_client.get_operation_state(operation_id=self.operation_id) 
    
    def wait_for_completion(self):
        """Wait for the operation to complete"""
        start_time = time()
        while self.state not in ('Succeeded', 'Failed'):
            self.state = self.get_operation_state()["status"]
            duration = int(time() - start_time)
            if duration > 60:
                
                if self.state == "Running":
                    self._logger.info(f"Operation did not complete after {duration} seconds")
                    return "Running"
                raise TimeoutError(f"Operation did not complete after {duration} seconds")
            sleep(3)
        return self.state
    

def check_long_running_operation(headers, core_client):
    """Check the status of a long-running operation"""
    location = headers.get('Location', None)
    operation_id = headers.get('x-ms-operation-id', None)
    if location:
        operation_id = location.split("/")[-1]
    
    logger_ = logger.getChild(__name__)

    if not operation_id:
        logger_.info("Operation initiated, no operation id found")
        return None
    lro = LongRunningOperation(operation_id=operation_id, core_client=core_client)
    lro.wait_for_completion()
    
    return lro.get_operation_results()